#include "atlas_1507_05493.h"
// AUTHOR: J.S. Kim
//  EMAIL: jsk@th.physik.uni-bonn.de
void Atlas_1507_05493::initialize() {
  setAnalysisName("atlas_1507_05493");          
  setInformation(""
    "# Search for photonic signatures of gauge-mediated supersymmetry in 8 TeV pp collisions with the ATLAS detector\n"
  "");
  setLuminosity(20.3*units::INVFB);      
  bookSignalRegions("gaga_SL;gaga_SH;gaga_WL;gaga_WH;gab_L;gab_H;gaj_L;gaj_H;gal_e;gal_mu");
  bookCutflowRegions("gaga_SL_photon;gaga_SL_etmiss;gaga_SL_HT;gaga_SL_meff,gaga_SL_Deltaphi(jet,etmiss)");

  // You can also book cutflow regions with bookCutflowRegions("CR1;CR2;..."). Note that the regions are
  //  always ordered alphabetically in the cutflow output files.

  // You should initialize any declared variables here
}

void Atlas_1507_05493::analyze() {
  // Your eventwise analysis code goes here
  // The following objects are always defined unless they are 'ignored' above. They form std::vector objects of the respective Delphes class type (except for Etmiss which is a single object)
  // All std::vector members and etmiss have the common properties PT, Eta, Phi and P4() with the latter giving access to the full ROOT TLorentzVector.
  // Within a std::vector, all members are ordered with highest pt coming first.

  // electronsLoose, electronsMedium, electronsTight   are list of electrons that passed respective efficiency and reconstruction cuts
  // muonsCombinedPlus, muonsCombined                  as above for muons
  // photonsMedium                                     as above for photons
  // jets are all reconstructed jets                   as above for jets. Note that electrons are most certainly also reconstructed as a jet -> overlap removal do avoid double counting necessary!
  // tracks, towers                                    calorimeter and tracker information. Usually not needed.
  // missingET                                         rec missing ET EXCLUDING muons.

  
  // Here is a couple of useful functions and lines:  
  //------------Phase Space Cuts (defined for jets, electronsXYZ, muonsXYZ, photonsXYZ)
  jets = filterPhaseSpace(jets, 20., -2.8, 2.8);  // The vector 'jets' only contains jets with pt >= 20 GeV and -2.8 < eta < 2.8. This function is applicable to other particles too (electronsMedium, ... ).
  electronsMedium = filterPhaseSpace(electronsMedium, 10., -2.47, 2.47);
  electronsTight = filterPhaseSpace(electronsTight, 10., -2.47, 2.47);

  muonsCombinedPlus = filterPhaseSpace(muonsCombinedPlus, 10., -2.5, 2.5);

  photons = filterPhaseSpace(photons, 20., -2.37, 2.37, true);
  //  photonsTight = filterPhaseSpace(photonsTight, 20., -2.37, 2.37, true);
  
  jets = overlapRemoval(jets, electronsTight, 0.2); // Removes all jets for which there exists any electron in 'electronsLoose' with deltaR < 0.2.
  // jets = overlapRemovel(jets, 0.2) If two jets overlap within deltaR < 0.2, only the harder jet is stored.
  //  jets = overlapRemoval(jets, photonsTight, 0.2);
  jets = overlapRemoval(jets, photons, 0.2);
  electronsTight = overlapRemoval(electronsTight,jets, 0.4);
  //  photonsTight = overlapRemoval(photonsTight,jets, 0.4);
  photons = overlapRemoval(photons,jets, 0.4);
  muonsCombined = overlapRemoval(muonsCombined,jets, 0.4);
  
  //------------Isolation Checks (defined for electronsXYZ, muonsXYZ, photonsXYZ
  //------------        For each object, if the user entered N isolation conditions, they can be
  //------------        checked individually be the second argument (running from 0 to N-1).
  // electronsMedium = filterIsolation(electronsMedium, 0)            Removes electrons that do not pass the first isolation condition entered into the AnalysisManager by the user
  // std::vector<int> flags; flags.push_back(0); flags.push_back(2);
  // electronsMedium = filterIsolation(electronsMedium, flags)        Same as above, but both the first and the third condition have to be fulfilled
  // electronsMedium = filterIsolation(electronsMedium)               Same as above, but all conditions have to be fulfilled.
  electronsTight = filterIsolation(electronsTight);

  std::vector<Photon*> photonstight = filterIsolation(photons,0);
  std::vector<Photon*> photonsloose = filterIsolation(photons,1);
  
  //-----------Flavour Tag Checks (defined for jets only)
  //----------          Tau tags "loose", "medium" or "tight" can be individually checked if the user enabled tau tagging in the AM.
  //----------          For b-tags, if N working points have been defined, the ith condition can be tested by putting i-1 into the second argument (if there is only one, the argument can be omitted)
  // if checkTauTag(jets[0], "tight") leadingJetIsTagged = True;
  // if checkBTag(jets[0], 0) leadingJetIsBTagged = True;//-----------Auxiliary Information
  // - Always ensure that you don't access vectors out of bounds. E.g. 'if(jets[1]->PT > 150)' should rather be if (jets.size() > 1 && jets[1]->PT > 150). 
  // - Use rand()/(RAND_MAX+1.) for random numbers between 0 and 1. The random seed is determined from system time or by the RandomSeed parameter in CheckMATE.
  // - The 'return' statement will end this function for the current event and hence should be called whenever the current event is to be vetoed.
  // - Many advanced kinematical functions like mT2 are implemented. Check the manual for more information.
  // - If you need output to be stored in other files than the cutflow/signal files we provide, check the manual for how to do this conveniently.  

  missingET->addMuons(muonsCombined);  // Adds muons to missing ET. This should almost always be done which is why this line is not commented out.

  //Photontrigger
  
  bool singlephotontrigger = false;
  bool doublephotontrigger = false;
    
  if( photonsloose.size()>=1 && photonsloose[0]->PT > 120. )
    singlephotontrigger = true;
  if ( photonsloose.size()>=2 && photonsloose[1]->PT > 40. )
    doublephotontrigger = true;

  std::vector<Jet*> bjets;
  for( int i=0; i<jets.size(); i++ ){
    if( checkBTag(jets[i]) )  bjets.push_back(jets[i]);
  }

  
  if( photonsloose.size() == 0 || !( singlephotontrigger || doublephotontrigger )  )
    return;


  //definitions of kinematical variables. Since Diphoton and single photon signal regions have different requirements on photon isolation, I define several kinematic observables for loose and tight photons

  //definition of effective mass (meff) and scalar mass (HT). Effective mass is equal to scalar mass + missing ET
  
  double meffloose=0.;
  double mefftight=0.;
  double HTloose=0.;
  double HTtight=0;
  
  for(int i=0; i<jets.size(); i++) {
    HTloose += jets[i]->PT;
  }
  
  for(int i=0; i<electronsTight.size(); i++) {
    HTloose += electronsTight[i]->PT;
  }

  for(int i=0; i<muonsCombined.size(); i++) {
   HTloose += muonsCombined[i]->PT;
  }

  HTtight=HTloose;
  
  for(int i=0; i<photonsloose.size(); i++) {
    HTloose += photonsloose[i]->PT;
  }

  meffloose = HTloose;
  meffloose += missingET->P4().Et();
  
  for(int i=0; i<photonstight.size(); i++) {
    HTtight += photonstight[i]->PT;
  }

  mefftight = HTtight;
  mefftight += missingET->P4().Et();

  // defintion of angular kinematic variables
  
  double deltaphi,deltaphi_j,deltaphi_ga_j;
  double tempphi=100.;

  //deltaphi_j will be defined later in the code

  //calculation of delta(gamma.MET)
  
  for ( int i=0; i<photonstight.size(); i++ ){
    deltaphi = fabs(photonstight[i]->P4().DeltaPhi(missingET->P4()));
    if( deltaphi < tempphi ) tempphi=deltaphi;
  }
  deltaphi=tempphi;
  
  //delta(gamma,jets) only the first two leading jets are taken into account
  
  for ( int i=0; i<jets.size(); i++ ){
    for ( int j=0; j<photonsloose.size(); j++ ){
      deltaphi_ga_j = fabs(jets[i]->P4().DeltaPhi(photonsloose[j]->P4()));
      if( deltaphi_ga_j < tempphi ) tempphi=deltaphi_ga_j;
       deltaphi_ga_j=tempphi;
    }
   
  }

  //definition of scalar sum of all jets and for the four leading jets
  
  double R4T=0.;

  double fourjets=0.;
  double alljets=0.;
  
  for(int i=0;i<jets.size();i++){
    alljets+=jets[i]->PT;
    if(i<5) fourjets +=jets[i]->PT;
  }

  //definition of the R4T variable
  
  if(jets.size()>3)
    R4T=fourjets/alljets;


  //definition of the transverse masses for gamma, electron and muon with missing transverse momentum. respectively
  
  double mTga,mTel,mTmu;

  
  //diphoton signal regions (table 2 in arxiv:1507.05493)
  
  if( photonstight.size() > 1 ){

    //construction of delta phi(jet,MET)
    
    tempphi=100.;

    if( jets.size() > 1 )
      for ( int i=0; i<jets.size(); i++ ){
	if( jets[i]->PT > 75. ) {
	  deltaphi_j = fabs(jets[i]->P4().DeltaPhi(missingET->P4()));
	  if( deltaphi_j < tempphi ) tempphi=deltaphi_j;
	}
      }
    else
      tempphi=100.;
  
    deltaphi_j=tempphi;


    
    if ( photonstight[0]->PT > 75. ){
      countCutflowEvent("gaga_SL_photon");
      if( missingET->P4().Et() > 150. ){
	countCutflowEvent("gaga_SL_etmiss");
	if( mefftight > 1800. ){
	  countCutflowEvent("gaga_SL_meff");
	  if( jets.size() > 1 && deltaphi_j > 0.5 ){
	    countCutflowEvent("gaga_Deltaphi(jet,etmiss)");
	    countSignalEvent("gaga_SL");
	  }
	}
      }
      if( missingET->P4().Et() > 250. )
	if( mefftight > 1500. )
	  if ( jets.size() > 1 && deltaphi_j > 0.5 )
	    if(deltaphi >0.5)
	      countSignalEvent("gaga_SH");

      if( missingET->P4().Et() > 150. )
	if( HTtight > 600. )
	  if ( jets.size() > 1 && deltaphi_j > 0.5 )
	    countSignalEvent("gaga_WL");

      if( missingET->P4().Et() > 200. )
	if( HTtight > 400. )
	  if ( jets.size() > 1 && deltaphi_j > 0.5 )
	    if( deltaphi > 0.5 )
	      countSignalEvent("gaga_WH");
    }

  }


  //single photon and one lepton signal regions  (table 4 in arxiv:1507.05493)

  
  if( photonsloose.size() > 0 && electronsTight.size() > 0 ){
    if( photonsloose[0]->PT > 125. ){
      if( missingET->P4().Et() > 120. && alljets < 100. )
	if( fabs( (photonsloose[0]->P4() + electronsTight[0]->P4() ).M() - 91. ) > 15. ){
	    mTel=mT(electronsTight[0]->P4(),missingET->P4(), 0.);
	    if( mTel > 120. )
	      if( photonsloose[0]->P4().DeltaR(electronsTight[0]->P4()) > 0.7 )
		countSignalEvent("gal_e");
	}
    }
  }

  if( photonsloose.size() > 0 && muonsCombined.size() > 0 ){
    if ( photonsloose[0]->PT > 125. ){
      if ( missingET->P4().Et() > 120. && alljets < 100. ){
	    mTmu=mT(muonsCombined[0]->P4(),missingET->P4(), 0.);
	    if( mTmu > 120. )
	      if( photons[0]->P4().DeltaR(muonsCombined[0]->P4()) > 0.7 )
		countSignalEvent("gal_mu");
      }
    }
    
    
  }
  
  //gamma jet signal regions with lepton veto   (table 3 in arxiv:1507.05493)
  
  if( photonsloose.size() > 0 && electronsTight.size() + muonsCombined.size() == 0 ){

    //construction of delta phi(jet,MET)
    
    tempphi=100.;

    if( jets.size() > 1 )
      for ( int i=0; i<jets.size(); i++ ){
	if( jets[i]->PT > 75. ) {
	  deltaphi_j = fabs(jets[i]->P4().DeltaPhi(missingET->P4()));
	  if( deltaphi_j < tempphi ) tempphi=deltaphi_j;
	}
      }
    else
      tempphi=100.;
  
    deltaphi_j=tempphi;

    
    if( photonsloose[0]->PT > 125. )
      if ( missingET->P4().Et() > 100. )
	if ( jets.size() > 1 && jets.size() < 5 && bjets.size() > 1 ){
	  double mbb;
	  mbb=( bjets[0]->P4() + bjets[1]->P4() ).M();
	  if( mbb > 75. && mbb < 150. ){
	    mTga = mT(photons[0]->P4(),missingET->P4(), 0.);
	    if( mTga > 90. )
	      if(deltaphi_j > 0.3 )
		countSignalEvent("gab_L");
	  }
	    
	}


    if( photonsloose[0]->PT > 125. )
      if ( missingET->P4().Et() > 200. )
	if ( jets.size() > 3  && jets[0]->PT > 100. && jets[1]->PT > 40. ){
	  if(deltaphi_j > 0.4 && R4T <0.85)
	    countSignalEvent("gaj_L");
	}
	    
    if( photonsloose[0]->PT > 300. )
      if ( missingET->P4().Et() > 300. && HTloose > 800. )
	if ( jets.size() > 1 && jets[0]->PT > 100. && jets[1]->PT > 40. ){
	  if( deltaphi_j > 0.4 && jets.size() > 1 )
	    if( deltaphi_ga_j < 2.0 )
	      countSignalEvent("gaj_H");
	}
    
  }


  //gamma jet with no lepton veto signal region  (table 3 in arxiv:1507.05493)
  
  if( photonsloose.size() > 0 ){

    //construction of delta phi(jet,MET)
    
      tempphi=100.;

      if( jets.size() > 3 )
	for ( int i=0; i<jets.size(); i++ ){
	  if( jets[i]->PT > 75. ) {
	    deltaphi_j = fabs(jets[i]->P4().DeltaPhi(missingET->P4()));
	    if( deltaphi_j < tempphi ) tempphi=deltaphi_j;
	  }
	}
      else
	tempphi=100.;
      
      deltaphi_j=tempphi;

    
    if ( photonsloose[0]->PT > 150. ){
      if ( missingET->P4().Et() > 200. && HTloose > 1000. )
	if( jets.size() > 3 && bjets.size() > 0){
	  mTga=mT(photons[0]->P4(),missingET->P4(), 0.);
	  if( mTga > 90. )
	    if( deltaphi_j > 0.3 && jets.size() > 3 )
	      countSignalEvent("gab_H");
	}
    }
  }


  
}

  
  

void Atlas_1507_05493::finalize() {
  // Whatever should be done after the run goes here
}       
